/**************************************************************************************************************************
// This script was authored by Lorne Brooks - Email: lbtools2k@gmail.com
//
//
// REVISION HISTORY:
// Version 1.1 19/08/23;
//						- Store Selection now stores object names to an INI data file (same as Selection Sets) for persistence.
//						- Added functionality to rename a selection set item and sort the selection sets list.
//						- Added thin bar button to open the INI data file for browsing or editing.
//						- Internal code refactoring and further UI tweaks.
// Version 1.01 12/08/23;
//						- Hotfix: prevented crash due to struct reference error.
// Version 1.0 11/08/23;
//						- First public release.
//
//
// This script is not copyrighted, please feel free to modify or distribute.
//
**************************************************************************************************************************/


global g_LBTools_quickSel

struct s_LBTools_quickSelection
(

/*==================================================================================
// Structure variables
====================================================================================*/

quickSelection_Rollout,
about_Rollout,
QS_pos,
qsINIfile = (getDir #plugcfg) + @"\LBTools_QuickSelection.ini",
textFile = @"C:\TEMP\LBTools_QuickSelection.txt",
imageGuide = (getDir #usericons) + @"\LBTools_Guide_Image.bmp",
imageNotepad = (getDir #usericons) + @"\LBTools_Notepad_Image.bmp",
pathsButtonState = false,
flag_hidden = false,
flag_frozen = false,
borderActive = bitMap 155 33 color:[0,250,180],
borderInactive = bitMap 155 33 color:gray,
pathsBorderState = borderInactive,
	
/*==================================================================================
// Structure functions
/*==================================================================================

f_initializeIniFile
f_createPathsTextFile
f_convertArrayToString
f_selectObjectsInScene
f_selectObjectsByItemType
f_selectPathObjects
f_quickSelection

====================================================================================*/

fn f_initializeIniFile =
(
	g_LBTools_quickSel.QS_pos = mouse.screenpos + [-75,-15]
	if NOT (doesFileExist g_LBTools_quickSel.qsINIfile) then
	(
		setIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "Position" (g_LBTools_quickSel.QS_pos as string)
		setIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "IncludeHidden" (g_LBTools_quickSel.flag_hidden as string)
		setIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "IncludeFrozen" (g_LBTools_quickSel.flag_frozen as string)
	)
	else
	(
		if NOT (hasIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "Position") then
		(
			setIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "Position" (g_LBTools_quickSel.QS_pos as string)
		)
		if NOT (hasIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "IncludeHidden") then
		(
			setIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "IncludeHidden" (g_LBTools_quickSel.flag_hidden as string)
		)
		if NOT (hasIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "IncludeFrozen") then
		(
			setIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "IncludeFrozen" (g_LBTools_quickSel.flag_frozen as string)
		)
	)
	g_LBTools_quickSel.QS_pos = execute (getIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "Position")
	g_LBTools_quickSel.flag_hidden = execute (getIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "IncludeHidden")
	g_LBTools_quickSel.flag_frozen = execute (getIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "IncludeFrozen")
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_createPathsTextFile =
(
	if NOT doesFileExist g_LBTools_quickSel.textFile then
	(
		makeDir @"C:\Temp"
		local edit_txtFile = openFile g_LBTools_quickSel.textFile mode:"w"
		format "*** EMPTY ***\n" to:edit_txtFile
		close edit_txtFile
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_convertArrayToString a_names =
(
	local ss_arrstr = stringstream ""
	append ss_arrstr "#("
	for n in a_names do
	(
		append ss_arrstr ("\"" + n + "\"" + ", ")
	)
	local arrstr = ss_arrstr as string
	arrstr = substring arrstr 1 (arrstr.count-2)
	arrstr = arrstr + ")"
	arrstr
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_selectObjectsInScene a_objs =
(
	local a_objsToSelect = #()
	
	for obj in a_objs do
	(
		if isValidNode obj then
		(
			local allowSelection = false
			
			if NOT obj.isHidden AND NOT obj.isFrozen then
			(
				allowSelection = true
			)
			else if obj.isHidden AND g_LBTools_quickSel.flag_hidden then
			(
				allowSelection = true
			)
			else if NOT obj.isHidden AND obj.isFrozen AND g_LBTools_quickSel.flag_frozen then
			(
				allowSelection = true
			)

			if allowSelection then append a_objsToSelect obj
		)
	)
	
	if a_objsToSelect.count > 0 then select a_objsToSelect
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_selectObjectsByItemType categoryName itemName =
(
	local a_objNames = execute (getIniSetting g_LBTools_quickSel.qsINIfile categoryName itemName)
	if a_objNames.count > 0 then
	(
		local a_setObjects = #()
		for objName in a_objNames do
		(
			local obj = getNodeByName objName exact:true
			if isValidNode obj then
			(
				append a_setObjects obj
			)
		)
		g_LBTools_quickSel.f_selectObjectsInScene a_setObjects
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_selectPathObjects item =
(
	if item != undefined AND item != "*** EMPTY ***" then
	(
		local ss_path = stringstream ""
		format "$%" item to:ss_path
		local str_path = ss_path as string
		--// Note: The onus is on the user to format their pathname literals correctly
		try(
			g_LBTools_quickSel.f_selectObjectsInScene (execute str_path)
		)catch(
			local ss_msg = stringstream ""
			format "Error: Pathname literal not valid for...\n\n%\n\nCheck that names with spaces are between single quotes." str_path to:ss_msg
			messagebox (ss_msg as string) title:"Error"
		)
	)
),

/*----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------*/

fn f_quickSelection =
(
	rollout quickSelection_Rollout "Quick Selection"
	(
		group "Settings"
		(
			checkbox chkbx_hidden "Select hidden objects" checked:(g_LBTools_quickSel.flag_hidden) tooltip:"Allow hidden objects to be included in the selection (frozen or not)"
			checkbox chkbx_frozen "Select visible frozen objects" checked:(g_LBTools_quickSel.flag_frozen) tooltip:"Allow visible frozen objects to be included in the selection"
			button btn_openINIfile "" width:70 height:8 align:#center tooltip:"Open the INI file for inspection"
			button btn_hideSelection "Hide" align:#center width:70 height:18 across:2 tooltip:"Set all selected objects to hidden"
			button btn_freezeSelection "Freeze" align:#center width:70 height:18 tooltip:"Set all selected objects to frozen"
			button btn_unhideSelection "Unhide" align:#center width:70 height:18 across:2 tooltip:"Set all selected objects to unhidden"
			button btn_unfreezeSelection "Unfreeze" align:#center width:70 height:18 tooltip:"Set all selected objects to unfrozen"
		)
		group "Category Selection"
		(
			button btn_geometry "Geometry" width:70 height:18 align:#left offset:[2,0] across:2
			button btn_shapes "Shapes" width:70 height:18 align:#right offset:[-2,0]
			button btn_lights "Lights" width:70 height:18 align:#left offset:[2,0] across:2
			button btn_cameras "Cameras" width:70 height:18 align:#right offset:[-2,0]
			button btn_helpers "Helpers" width:70 height:18 align:#left offset:[2,0] across:2
			button btn_bones "Bones" width:70 height:18 align:#right offset:[-2,0]
		)
		group "Stored Selection"
		(
			button btn_storeSelection "Store" align:#center width:152 height:18 tooltip:"Store selected objects so they can be grabbed later"
			bitmap borderImageStored bitMap:(g_LBTools_quickSel.borderInactive)
			button btn_grabStoredObjs "Grab Stored Objects" enabled:false align:#center width:145 height:25 offset:[0,-40] tooltip:"Select the objects that were stored"
			label llb_gap1 "" offset:[0,-8]
		)
		group "Selection Sets"
		(
			edittext edtxt_setName "Name: " align:#left fieldwidth:115 tooltip:"Enter a name for a new selection set to store"
			button btn_addSet "Add" ailgn:#left width:70 height:18 across:2 offset:[-2,0] tooltip:"Store selected objects as a new selection set using the name provided"
			button btn_deleteSet "Delete" align:#right width:70 height:18 offset:[-4,0] tooltip:"Removes the picked selection set from the list"
			button btn_renameSet "Rename" ailgn:#left width:70 height:18 across:2 offset:[-2,0] tooltip:"Rename the picked selection set using the entered name provided"
			button btn_sortSets "Sort" align:#right width:70 height:18 offset:[-4,0] tooltip:"Sort the selection set names alphabetically"
			dropdownlist ddl_sets "" items:#() align:#center width:150 height:20 tooltip:"Choose a selection set to select objects from"
			bitmap borderImageSets bitMap:(g_LBTools_quickSel.borderInactive)
			button btn_grabSetObjs "Grab Set Objects" enabled:false ailgn:#center width:145 height:25 offset:[0,-40] tooltip:"Select the objects for the picked selection set"
			label llb_gap2 "" offset:[0,-8]
		)
		group "Path Selection"
		(
			button btn_refresh "Refresh List" align:#left width:100 height:18 offset:[25,0] across:2 tooltip:"Refresh the list with text file contents"
			button btn_openFile "" images:#(g_LBTools_quickSel.imageNotepad, undefined, 1, 1, 1, 1, 1, false) align:#right width:18 height:20 offset:[-6,0] tooltip:"Open a text file in an editor to add path strings to the list"
			label lbl_path "$" align:#left offset:[2,4] across:2
			dropdownlist ddl_paths "" items:#() align:#right width:147 height:20 offset:[0,0] tooltip:"Choose a path definition to select objects from"
			bitmap borderImagePaths bitMap:(g_LBTools_quickSel.pathsBorderState)
			button btn_grabPathObjs "Grab Path Objects" enabled:(g_LBTools_quickSel.pathsButtonState) align:#center width:145 height:25 offset:[0,-40] tooltip:"Select the objects for the current paths item"
			label llb_gap3 "" offset:[0,-8]
		)
		button btn_about "About" align:#center width:40 height:20
		
		fn f_loadSelectionSets =
		(
			if (hasIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets") then
			(
				ddl_sets.items = getIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets"
			)
			else
			(
				ddl_sets.items = #()
			)
		)

		fn f_loadPathItemsFromFile =
		(
			local a_pathItems = #()
			local textFileRead = openFile g_LBTools_quickSel.textFile mode:"r"
			while NOT eof textFileRead do
			(
				local str_line = readLine textFileRead
				if str_line != "" then --// Ignore blank lines
				(
					append a_pathItems str_line
				)
			)
			close textFileRead
			if a_pathItems.count == 0 then
			(
				ddl_paths.items = #("*** EMPTY ***")
				btn_grabPathObjs.enabled = g_LBTools_quickSel.pathsButtonState = false
				borderImagePaths.bitMap = g_LBTools_quickSel.pathsBorderState = g_LBTools_quickSel.borderInactive
			)
			else
			(
				ddl_paths.items = a_pathItems
				btn_grabPathObjs.enabled = g_LBTools_quickSel.pathsButtonState = true
				borderImagePaths.bitMap = g_LBTools_quickSel.pathsBorderState = g_LBTools_quickSel.borderActive
				if a_pathItems.count == 1 AND a_pathItems[1] == "*** EMPTY ***" then
				(
					btn_grabPathObjs.enabled = g_LBTools_quickSel.pathsButtonState = false
					borderImagePaths.bitMap = g_LBTools_quickSel.pathsBorderState = g_LBTools_quickSel.borderInactive
				)
			) 	
		)
		
		on quickSelection_Rollout open do
		(
			--// Get stored paths for dropdown list from text file
			f_loadPathItemsFromFile()
			
			--// Get stored selection sets for dropdown list from INI file
			f_loadSelectionSets()
			
			if ddl_sets.items.count > 0 then
			(
				btn_grabSetObjs.enabled = true
				borderImageSets.bitmap = g_LBTools_quickSel.borderActive
			)
			else
			(
				btn_grabSetObjs.enabled = false
				borderImageSets.bitmap = g_LBTools_quickSel.borderInactive
			)
			
			if (hasIniSetting g_LBTools_quickSel.qsINIfile "Stored" "Items") then
			(
				btn_grabStoredObjs.enabled = true
				borderImageStored.bitmap = g_LBTools_quickSel.borderActive
			)
			else
			(
				btn_grabStoredObjs.enabled = false
				borderImageStored.bitmap = g_LBTools_quickSel.borderInactive
			)
		)
			
		on quickSelection_Rollout moved pos do
		(
			g_LBTools_quickSel.QS_Pos = pos
			setIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "Position" (g_LBTools_quickSel.QS_pos as string)
		)
		
		on quickSelection_Rollout close do
		(
			try(destroyDialog g_LBTools_quickSel.about_Rollout)catch()
		)
		
		on btn_about pressed do
		(
			try(destroyDialog g_LBTools_quickSel.about_Rollout)catch()
			createDialog g_LBTools_quickSel.about_Rollout
		)
		
		on btn_openINIfile pressed do
		(
			ShellLaunch g_LBTools_quickSel.qsINIfile ""
		)
		
		on btn_geometry pressed do
		(
			local a_geometry = for g in geometry where classof g != BoneGeometry collect g
			g_LBTools_quickSel.f_selectObjectsInScene a_geometry
		)
		on btn_shapes pressed do g_LBTools_quickSel.f_selectObjectsInScene shapes
		on btn_lights pressed do g_LBTools_quickSel.f_selectObjectsInScene lights
		on btn_cameras pressed do g_LBTools_quickSel.f_selectObjectsInScene cameras
		on btn_helpers pressed do g_LBTools_quickSel.f_selectObjectsInScene helpers
		on btn_bones pressed do
		(
			local a_bones = for o in objects where classof o == BoneGeometry collect o
			g_LBTools_quickSel.f_selectObjectsInScene a_bones
		)

		on chkbx_hidden changed state do
		(
			g_LBTools_quickSel.flag_hidden = state
			setIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "IncludeHidden" (g_LBTools_quickSel.flag_hidden as string)
		)
		
		on chkbx_frozen changed state do
		(
			g_LBTools_quickSel.flag_frozen = state
			setIniSetting g_LBTools_quickSel.qsINIfile "QuickSelection" "IncludeFrozen" (g_LBTools_quickSel.flag_frozen as string)
		)
		
		on btn_hideSelection pressed do
		(
			undo "HideSelection" on hide selection
		)
		
		on btn_unhideSelection pressed do
		(
			undo "UnhideSelection" on unhide selection
		)
		
		on btn_freezeSelection pressed do
		(
			undo "FreezeSelection" on freeze selection
		)
		
		on btn_unfreezeSelection pressed do
		(
			undo "UnfreezeSelection" on unfreeze selection
		)
		
		--// STORED SELECTION
		on btn_storeSelection pressed do
		(
			if selection.count > 0 then
			(
				local a_objNames = for obj in selection collect obj.name
				local arrstr = f_convertArrayToString a_objNames
				setIniSetting g_LBTools_quickSel.qsINIfile "Stored" "Items" arrstr
				btn_grabStoredObjs.enabled = true
				borderImageStored.bitmap = g_LBTools_quickSel.borderActive
			)
			else
			(
				messageBox "Error: Select some objects to store first." title:"Error"
			)
		)
				
		on btn_grabStoredObjs pressed do
		(
			if (hasIniSetting g_LBTools_quickSel.qsINIfile "Stored" "Items") then
			(
				local a_objNames = getIniSetting g_LBTools_quickSel.qsINIfile "Stored" "Items"
				if a_objNames.count > 0 then
				(
					g_LBTools_quickSel.f_selectObjectsByItemType "Stored" "Items"
				)
				else
				(
					messageBox "Error: Nothing is stored." title:"Error"
				)
			)
		)

		--// STORED SELECTION SETS
		on btn_addSet pressed do
		(
			if edtxt_setName.text != "" then
			(
				local a_filtered = filterstring edtxt_setName.text " "
				if a_filtered.count > 0 then
				(					
					if selection.count > 0 then
					(
						if (hasIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets" edtxt_setName.text) then
						(
							if NOT (querybox "Selection set name already exists.\n\nDo you want to update it using the currently selected objects?" title:"Quick Selection") then
							(
								--// Abort replacing objects in the set
								return undefined
							)
						)
						local a_objNames = for obj in selection collect obj.name
						--// Convert the array to a string to avoid the long array truncation with "..." for INI file storage
						local arrstr = f_convertArrayToString a_objNames
						setIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets" edtxt_setName.text arrstr
						f_loadSelectionSets()
						ddl_sets.selection = ddl_sets.items.count
						btn_grabSetObjs.enabled = true
						borderImageSets.bitmap = g_LBTools_quickSel.borderActive
					)
					else
					(
						messagebox "Error: Must have one or more objects selected." title:"Error"
						setFocus edtxt_setName
					)
				)
				else
				(
					messagebox "Error: Must enter a selection set name (and no white space only)." title:"Error"
					edtxt_setName.text = ""
					setFocus edtxt_setName
				)
			)
			else
			(
				messagebox "Error: Must enter a selection set name." title:"Error"
				setFocus edtxt_setName
			)
		)
		
		on btn_deleteSet pressed do
		(
			if ddl_sets.items.count > 0 then
			(
				if (querybox "Are you sure you want to delete the selection set?" title:"Quick Selection") then
				(
					local sets = getIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets"
					delIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets" sets[ddl_sets.selection]
					f_loadSelectionSets()
					if ddl_sets.items.count > 0 then
					(
						ddl_sets.selection = 1
					)
					else
					(
						btn_grabSetObjs.enabled = false
						borderImageSets.bitmap = g_LBTools_quickSel.borderInactive
					)
				)
			)
			else
			(
				messagebox "Error: No selection sets available." title:"Error"
			)
		)
		
		on btn_renameSet pressed do
		(
			if ddl_sets.items.count > 0 then
			(
				if edtxt_setName.text != "" then
				(
					local a_filtered = filterstring edtxt_setName.text " "
					if a_filtered.count > 0 then
					(
						if edtxt_setName.text != ddl_sets.selected then
						(
							local a_objNameStrings = getIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets" ddl_sets.selected
							delIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets" ddl_sets.selected
							setIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets" edtxt_setName.text a_objNameStrings
							f_loadSelectionSets()
							ddl_sets.selection = ddl_sets.items.count
						)
						else
						(
							messagebox "Error: Selection set name already exists." title:"Error"
							edtxt_setName.text = ""
							setFocus edtxt_setName						
						)
					)
					else
					(
						messagebox "Error: Must enter a selection set name (and no white space only)." title:"Error"
						edtxt_setName.text = ""
						setFocus edtxt_setName
					)
				)
				else
				(
					messagebox "Error: Must enter a selection set name." title:"Error"
					setFocus edtxt_setName
				)
			)
			else
			(
				messagebox "Error: No selection sets available." title:"Error"
			)
		)
		
		on btn_sortSets pressed do
		(
			if ddl_sets.items.count > 0 then
			(
				--// Collect a mapping of the existing sets
				local a_sets = #()
				local a_setNames = getIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets"
				for setName in a_setNames do
				(
					append a_sets #(setName, (getIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets" setName))
				)
				
				--// Build a new sorted array of sets
				local a_sortedSets = #()
				local a_sortedSetNames = sort(getIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets")
				for i = 1 to a_sortedSetNames.count do
				(
					for j = 1 to a_sets.count do
					(
						if a_sortedSetNames[i] == a_sets[j][1] then
						(
							append a_sortedSets a_sets[j]
							exit
						)
					)
				)
				
				--// Delete the old sets from the INI file
				delIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets"
				
				--// Add the new sorted sets to the INI file
				for a_sortedSet in a_sortedSets do
				(
					setIniSetting g_LBTools_quickSel.qsINIfile "SelectionSets" a_sortedSet[1] a_sortedSet[2]
				)
				
				f_loadSelectionSets()
				ddl_sets.selection = 1
			)
			else
			(
				messagebox "Error: No selection sets available." title:"Error"
			)
		)
		
		on ddl_sets selected val do
		(
			g_LBTools_quickSel.f_selectObjectsByItemType "SelectionSets" ddl_sets.items[val]
		)
		
		on btn_grabSetObjs pressed do
		(
			g_LBTools_quickSel.f_selectObjectsByItemType "SelectionSets" ddl_sets.items[ddl_sets.selection]
		)
		
		--// PATH SELECTION
		on btn_openFile pressed do
		(
			--// Open a text file to add string items to use in the dropdown list
			if doesFileExist g_LBTools_quickSel.textFile then
			(
				ShellLaunch g_LBTools_quickSel.textFile ""
			)
			else
			(
				messagebox "Error: Text file does not exist. Recreating it." title:"Error"
				g_LBTools_quickSel.f_createPathsTextFile()
				f_loadPathItemsFromFile()
			)
		)
		
		on btn_refresh pressed do
		(
			--// Update the items list from the text file (assume they are each listed on a new line)
			if doesFileExist g_LBTools_quickSel.textFile then
			(
				f_loadPathItemsFromFile()
			)
			else
			(
				messagebox "Error: Text file does not exist. Recreating it." title:"Error"
				g_LBTools_quickSel.f_createPathsTextFile()
				f_loadPathItemsFromFile()
			)
		)
		
		on btn_grabPathObjs pressed do
		(
			g_LBTools_quickSel.f_selectPathObjects ddl_paths.selected
		)
		
		on ddl_paths selected val do
		(
			g_LBTools_quickSel.f_selectPathObjects ddl_paths.items[val]
		)		
	) --// end rollout
	
	rollout About_Rollout "About"
	(
		label lbl_author "Author - Lorne Brooks"
		hyperLink myEmail "lbtools2k@gmail.com" across:2 align:#center offset:[15,10] color:(color 0 0 255) hovercolor:(color 0 0 190) visitedcolor:(color 190 0 0) align:#center address:"mailto:lbtools2k@gmail.com"
		button btn_guide "" align:#right offset:[0,0] images:#(g_LBTools_quickSel.imageGuide, undefined, 1, 1, 1, 1, 1, false) tooltip:"Opens an online guide explaining the tool's features."
		label lbl_version "Version 1.1 - 19 Aug 2023"
		
		on btn_guide pressed do
		(
			shellLaunch "https://sites.google.com/view/lbtools-guide/home/quick-selection" ""
		)
	) --// end rollout
	
	-----------------------------------------------------------------------------------------
	--// Setup Interface
	-----------------------------------------------------------------------------------------
	--// initialize the script INI file for persistent data
	g_LBTools_quickSel.f_initializeIniFile()
	
	--// Create the paths text file if it doesn't exist
	g_LBTools_quickSel.f_createPathsTextFile()
	
	fn f_openQuickSelectionUI =
	(
		try(destroyDialog quickSelection_Rollout)catch()
		createDialog quickSelection_Rollout width:185 pos:g_LBTools_quickSel.QS_pos
	)
	
	f_openQuickSelectionUI()
)

) --// end struct

g_LBTools_quickSel = s_LBTools_quickSelection()


/*********************************************************************************************************************************************************************/
--// MACRO CALLS FOR UI
/*********************************************************************************************************************************************************************/

macroscript LBTools_quickSelection
	category:"LB Tools"
	icon:#("LBTools_QuickSelection",1)
	buttontext:"{ QS }"
	tooltip:"Quick Selection 1.1"
(
	g_LBTools_quickSel.f_quickSelection()
)

/*********************************************************************************************************************************************************************/